from codemie_tools.base.models import ToolMetadata

IMAGE_TOOL = ToolMetadata(
    name="image_tool",
    description="Tool for interpreting and transcribing already initialized image contents",
    label="Image Recognition",
)
